@@ -1,3 +1,6 @@
+'''This class offers basic plot Functions for generating nice heatmaps.
+'''
+
 from scipy.spatial.distance import pdist
 from scipy.cluster.hierarchy import linkage, dendrogram, cut_tree
 import numpy as np
@@ -41,80 +44,86 @@ def Heatmap(table,
         optimal_row_ordering = True,
         optimal_col_ordering = True,
         ax = None):
-    """
-        Function that plots a two dimensional matrix as clustered heatmap.
-        Sorting of rows and columns is done by hierarchical clustering.
+    """Function that plots a two dimensional matrix as clustered heatmap.
+    Sorting of rows and columns is done by hierarchical clustering.
 
-        args:
-            table: pandas.DataFrame
-                Two dimensional array containing numerical values to be
-                clustered.
-        kwargs:
-            cmap: str
-                Colormap used to produce color scale.
-            distance_metric: str
-                Distance metric used to determine distance between two vectors.
-                The distance function can be either of 'braycurtis', 'canberra',
-                'chebyshev', 'cityblock', 'correlation', 'cosine', 'dice',
-                'euclidean', 'hamming', 'jaccard', 'jensenshannon', 'kulsinski',
-                'mahalanobis', 'matching', 'minkowski', 'rogerstanimoto',
-                'russellrao', 'seuclidean', 'sokalmichener', 'sokalsneath',
-                'sqeuclidean', 'yule'.
-            linkage_method: str
-                methods for calculating the distance between the newly formed
-                cluster u and each v. Possible methods are, 'single',
-                'complete', 'average', 'weighted', and 'centroid'
-            show_column_labels: bool
-                If true, show labels of columns, as defined in table.
-            show_row_labels: bool
-                If true, show labels of rows, as defined in table
-            row_clustering: bool
-                If true, the rows a clustered according to distance_metric, and
-                linkage_method.
-            column_clustering; bool
-                If true, the columns are clustered according to distance:metric,
-                and linkage_method.
-            custom_row_clustering: list
-                List of Row ids from table in the order they should appear in
-                the heatmap. Only applies if row_clustering is False.
-            custom_column_clustering: list
-                List of column ids from table in the order they should appear
-                in the heatmap.    Only applies if column_clustering is False.
-            vmin: float
-                Minimal value of data_table, that has a color representation.
-            vmax: float
-                Maximal value of data_table, that has a color representation.
-            symmetric_color_scale: bool
-                If true, vmin, and vmax will be set to have equal distance from
-                symmetry point.
-            symmetry_point: float
-                Only used, when symmetric_color_scale is true. If
-                symmetric_color_scale is true, and symmetry_point is not set,
-                it defaults to zero.
-            optimal_row_ordering: boolean
-                If True, the rows will be ordered optimally with regards to
-                the cluster separation. Be careuful: Can take a long time,
-                depending on the number of rows.
-            optimal_col_ordering: boolean
-                If True, the columns will be ordered optimally with regards to
-                the cluster separation. Be careuful: Can take a long time,
-                depending on the number of columns.
-            ax: matplotlib.axes.Axes
-                Axes instance on which to plot heatmap.
-        Returns: tuple
-            column_names_reordered: list
-                List of column names after reordering as given as column names
-                of table.
-            row_names_reordered: list
-                List of row names after reordering as given as index names of
-                table.
-            vmin: float
-                Minimal value of table, that gets a color representation in
-                heatmap.
-            vmax: float
-                Maximal value of table, that gets a color representation in
-                heatmap.
+    :param table: Two dimensional array containing numerical values to be
+        clustered.
+    :type table: Object of type :class:`pandas.DataFrame`
+    :param cmap: Colormap used to produce color scale, defaults to "Reds".
+    :type cmap: str, optional
+    :param distance_metric: Distance metric used to determine distance
+        between two vectors.The distance function can be either of
+        'braycurtis', 'canberra', 'chebyshev', 'cityblock', 'correlation',
+        'cosine', 'dice', 'euclidean', 'hamming', 'jaccard',
+        'jensenshannon', 'kulsinski', 'mahalanobis', 'matching',
+        'minkowski', 'rogerstanimoto', 'russellrao', 'seuclidean',
+        'sokalmichener', 'sokalsneath', 'sqeuclidean', 'yule', defaults to
+        'correlation'.
+    :type distance_metric: str, optional
+    :param linkage_method: methods for calculating the distance between the
+        newly formed cluster u and each v. Possible methods are, 'single',
+        'complete', 'average', 'weighted', and 'centroid', defaults to
+        'complete'.
+    :type linkage_method: str, optional
+    :param show_column_labels: If true, show labels of columns, as defined
+        in table, defaults to False.
+    :type show_column_labels: bool, optional
+    :param show_row_labels: If true, show labels of rows, as defined in
+        table, defaults to False.
+    :type show_row_labels: bool, optional
+    :param row_clustering: If true, the rows a clustered according to
+        distance_metric, and linkage_method, defaults to True.
+    :type row_clustering: bool, optional
+    :param column_clustering: If true, the columns are clustered according
+        to distance_metric, and linkage_method, defaults to True.
+    :type column_clustering: bool, optional
+    :param custom_row_clustering: List of Row ids from table in the order
+        they should appear in the heatmap. Only applies if row_clustering is
+        False, defaults to None.
+    :type custom_row_clustering: list, optional
+    :param custom_column_clustering: List of column ids from table in the
+        order they should appear in the heatmap. Only applies if
+        column_clustering is False, defaults to None.
+    :type custom_column_clustering: list, optional
+    :param vmin: Minimal value of data_table, that has a color
+        representation, defaults to None.
+    :type vmin: float, optional
+    :param vmax: Maximal value of data_table, that has a color
+        representation, defaults to None.
+    :type vmax: float, optional
+    :param symmetric_color_scale: If true, vmin, and vmax will be set to
+        have equal distance from symmetry point, defaults to False.
+    :type symmetric_color_scale: bool, optional
+    :param symmetry_point: Only used, when symmetric_color_scale is true. If
+        symmetric_color_scale is true, and symmetry_point is not set,
+        defaults to zero.
+    :type symmetry_point: float, optional
+    :param show_plot: If True, the heatmap will be shown on the given axis,
+        else the function only returns the return values, defaults to True.
+    :type show_plot: bool, optional
+    :param optimal_row_ordering: If True, the rows will be ordered optimally
+        with regards to the cluster separation. Be careuful: Can take a
+        long time, depending on the number of rows, defaults to True.
+    :type optimal_row_ordering: bool, optional
+    :param optimal_col_ordering: If True, the columns will be ordered
+        optimally with regards to the cluster separation. Be careuful:
+        Can take a long time, depending on the number of columns, defaults
+        to True.
+    :type optimal_col_ordering: bool, optional
+    :param ax: Axes instance on which to plot heatmap, defaults to None.
+    :type ax: :class:`matplotlib.axes._subplots.AxesSubplot`,
+        optional
 
+    :return: Tuple containing: 1. column_names_reordered: List of
+        column names after reordering as given as column names of table.
+        2. row_names_reordered: List of row names after reordering as given
+        as index names of table.
+        3. vmin: Minimal value of table, that gets a color representation in
+        heatmap.
+        4. vmax: Maximal value of table, that gets a color representation in
+        heatmap.
+    :rtype: tuple
     """
     if(show_plot):
         ax = ax if ax is not None else plt.gca()
@@ -123,7 +132,7 @@ def Heatmap(table,
     column_names_reordered = list(table.columns)
     if(column_clustering):
         distance_matrix = pdist(table.T, metric=distance_metric)
-        linkage_matrix = linkage(distance_matrix, 
+        linkage_matrix = linkage(distance_matrix,
                                  metric=distance_metric,
                                  method=linkage_method,
                                  optimal_ordering=optimal_col_ordering)
@@ -140,7 +149,7 @@ def Heatmap(table,
     row_names_reordered = list(table.index)
     if(row_clustering):
         distance_matrix = pdist(table, metric=distance_metric)
-        linkage_matrix = linkage(distance_matrix, 
+        linkage_matrix = linkage(distance_matrix,
                                  metric=distance_metric,
                                  method=linkage_method,
                                  optimal_ordering=optimal_row_ordering)
@@ -181,7 +190,7 @@ def Heatmap(table,
         plt.ylim(-0.5, nrows-.5)
         plt.xlim(-0.5, ncols-.5)
         img.set_rasterized(True)
-    
+
         # Plot column/ row labels
         if(show_column_labels):
             plt.xticks([ i for i in range(len(column_names_reordered))],
@@ -206,46 +215,54 @@ def Dendrogram(table,
         optimal_row_ordering=True,
         optimal_col_ordering=True,
         ax = None):
-    """
-        Function that plots a dendrogram on on axis 0 (rows), or axis 1
-        (columns) of a pandas.DataFrame.
+    """Function that plots a dendrogram on axis 0 (rows), or axis 1
+    (columns) of a :class:`pandas.DataFrame`.
 
-        args:
-            table: pandas.DataFrame
-                Data matrix used to calculate dendrograms.
-        kwargs:
-            distance_metric: str
-                Distance metric used to determine distance between two vectors.
-                The distance function can be either of 'braycurtis', 'canberra',
-                'chebyshev', 'cityblock', 'correlation', 'cosine', 'dice',
-                'euclidean', 'hamming', 'jaccard', 'jensenshannon', 'kulsinski',
-                'mahalanobis', 'matching', 'minkowski', 'rogerstanimoto',
-                'russellrao', 'seuclidean',    'sokalmichener', 'sokalsneath',
-                'sqeuclidean', 'yule'.
-            linkage_method: str
-                methods for calculating the distance between the newly formed
-                cluster u and each v. Possible methods are, 'single',
-                'complete', 'average', 'weighted', and 'centroid'
-            axis: int
-                Axis of table used for plotting dendrogram (0 = rows,
-                1 = columns).
-            lw: float
-                width of the lines (in points) defining the dengrogram.
-            optimal_row_ordering: boolean
-                If True, the rows will be ordered optimally with regards to
-                the cluster separation. Be careuful: Can take a long time,
-                depending on the number of rows.
-            optimal_col_ordering: boolean
-                If True, the columns will be ordered optimally with regards to
-                the cluster separation. Be careuful: Can take a long time,
-                depending on the number of columns.
-            ax: matplotlib.axes.Axes
-                Axes n which to plot the dendrogram.
-        Returns:
-            dendrogram_dict: dict
-                Resulting dictionary from scipy.cluster.hierarchy.dendrogram
-                function.
+    :param table: Data matrix used to calculate dendrograms.
+    :type table: :class:`pandas.DataFrame`
+    :param distance_metric: Distance metric used to determine distance between
+        two vectors. The distance function can be either of 'braycurtis',
+        'canberra', 'chebyshev', 'cityblock', 'correlation', 'cosine', 'dice',
+        'euclidean', 'hamming', 'jaccard', 'jensenshannon', 'kulsinski',
+        'mahalanobis', 'matching', 'minkowski', 'rogerstanimoto', 'russellrao', 
+        'seuclidean', 'sokalmichener', 'sokalsneath', 'sqeuclidean', 'yule',
+        defaults to 'correlation'.
+    :type distance_metric: str, optional
+    :param linkage_method: Methods for calculating the distance between the
+        newly formed cluster u and each v. Possible methods are, 'single',
+        'complete', 'average', 'weighted', and 'centroid', defaults to
+        'complete'.
+    :type linkage_method: str, optional
+    :param axis: Axis of table used for plotting dendrogram (0 = rows,
+        1 = columns), defaults to 1.
+    :type axis: int, optional
+    :param lw: Width of the lines (in points) defining the dengrogram, defaults
+        to 1.
+    :type lw: float, optional
+    :param n_clust: Number of clusters in which the dendrogram should be cut.
+        Note: Dendrogram cutting makes use of scipy.cluster.hierarchy.cut_tree
+        function, defaults to None.
+    :type n_clust: int, optional
+    :param optimal_row_ordering: If True, the rows will be ordered optimally
+        with regards to the cluster separation. Be careuful: Can take a long
+        time, depending on the number of rows, defaults to True.
+    :type optimal_row_ordering: bool, optional
+    :param optimal_col_ordering: If True, the columns will be ordered optimally 
+        with regards to the cluster separation. Be careuful: Can take a long
+        time, depending on the number of columns, defaults to True.
+    :type optimal_col_ordering: bool, optional
+    :param ax: Axes n which to plot the dendrogram, defaults to None.
+    :type ax: :class:`matplotlib.axes._subplots.AxesSubplot`
 
+    :return: Tuple containing the following entries:
+        1. Resulting dictionary from scipy.cluster.hierarchy.dendrogram
+        function.
+        2. linkage matrix, that is returned from
+        scipy.cluster.hierarchy.linkage function
+        3. dictionary containing the row_ids, (in case of axis = 0), or
+        column_ids (in case of axis = 1) that are assigned to different
+        clusters. Only makes sense if n_clust is not None.
+    :rtype: dict
     """
     ax = ax if ax is not None else plt.gca()
 
@@ -257,7 +274,7 @@ def Dendrogram(table,
     if(axis == 0):
         ids = list(table.index)
         distance_matrix = pdist(table, metric=distance_metric)
-        linkage_matrix = linkage(distance_matrix, 
+        linkage_matrix = linkage(distance_matrix,
                                  metric=distance_metric,
                                  method=linkage_method,
                                  optimal_ordering=optimal_row_ordering)
@@ -273,7 +290,7 @@ def Dendrogram(table,
                     cluster_dict[group] += [ids[i]]
         with plt.rc_context({'lines.linewidth': lw}):
             dendrogram_dict = dendrogram(linkage_matrix,
-                                         orientation="left", 
+                                         orientation="left",
                                          color_threshold = color_threshold)
     elif(axis == 1):
         ids = table.columns
@@ -300,48 +317,62 @@ def Dendrogram(table,
     ax.spines["right"].set_visible(False)
     ax.spines["bottom"].set_visible(False)
     ax.spines["top"].set_visible(False)
-    
+
     plt.xticks([], [])
     plt.yticks([], [])
 
     return dendrogram_dict, linkage_matrix, cluster_dict
 
-def Annotation(ids_sorted, annotation_df, annotation_col_id, axis = 1,
-               color_list = colors["xkcd"], is_categorial=True,
-               cmap = plt.cm.GnBu_r, color_dict = None, ax = None):
-    """
-        Function that plots annotations.
+def Annotation(ids_sorted,
+               annotation_df,
+               annotation_col_id,
+               axis = 1,
+               color_list = colors["xkcd"],
+               is_categorial=True,
+               cmap = plt.cm.GnBu_r,
+               color_dict = None,
+               ax = None):
+    """Function that plots annotations.
 
-        args:
-            ids_sorted: list
-                List of ids in the order in which the annotation shall be
-                plotted.
-            annotation_df: pandas.DataFrame
-                DataFrame containing grouping informatation for the ids for
-                which the annotation shall be plotted. Columns: groups, rows:
-                ids.
-            annotation_col_id: str
-                the column id of the group in annotation_df, for which the
-                annotation mshall be plotted.
-        kwargs:
-            axis: int
-                If 0, then the annotation is plotted vertically, i.e. for rows
-                of a DataFrame. If 1, then the annotation is plotted
-                horizontally, i.e. for columns of a DataFrame.
-            color_list: list
-                List of colors used to plot annotations.
-            is_categorial: boolean
-                Boolean parameter that defines if the colorscale shall be
-                categorial, or continuous (e.g. age).
-            cmap: matplotlib.pyplot.cm
-                ColorMap object, that defines the colormap used for plotting 
-                continuous variables.
-            ax: matplotlib.axes.Axes
-                Axes on which to plot the annotation.
-        Returns:
-            patch_list:
-                List containing one representative patch per group, along with
-                the group name and the color representation of the group,
+    :param ids_sorted: List of ids in the order in which the annotation shall be
+        plotted.
+    :type ids_sorted: list
+    :param annotation_df: DataFrame containing grouping informatation for the
+        ids for which the annotation shall be plotted. Columns: groups, rows:
+        ids.
+    :type annotation_df: :class:`pandas.DataFrame`
+    :param annotation_col_id: Column id of the group in annotation_df, for which
+        the annotation mshall be plotted.
+    :type annotation_col_id: str
+    :param axis: If 0, then the annotation is plotted vertically, i.e. for rows
+        of a DataFrame. If 1, then the annotation is plotted horizontally, i.e.
+        for columns of a DataFrame, defaults to 1
+    :type axis: int, optional
+    :param color_list: List of colors used to plot annotations, defaults to
+        colors["xkcd"].
+    :type color_list: list, optional
+    :param is_categorial: Boolean parameter that defines if the colorscale shall
+        be categorial, or continuous (e.g. age), defaults to True
+    :type is_categorial: bool, optional
+    :param cmap: ColorMap object, that defines the colormap used for plotting
+        continuous variables, defaults to plt.cm.GnBu_r.
+    :type cmap: :class:`matplotlib.colors.LinearSegmentedColormap`, optional
+    :param color_dict: If is_categorial is True you can define colors for each
+        category. These colors have to be given as a dict, where the key is the
+        category and the value is the color code, defaults to None.
+    :type color_dict: dict, optional
+    :param ax: Axes on which to plot the annotation, defaults to None.
+    :type ax: class:`matplotlib.axes._subplots.AxesSubplot`, optional
+
+    :return: tuple containing the following two elements:
+        1. True, if the annotation was categorial, False otherwise.
+        2. If the first element is True, then it is a list containing one
+        representative patch per group, along with the group name and the color
+        representation of the group. If the first element is False, then it is
+        a list, containing the
+        :class:`matplotlib.colors.LinearSegmentedColormap` object, the min, and
+        the max value from the floating value annotations.
+    :rtype: tuple
     """
     ax = ax if ax is not None else plt.gca()
 
@@ -349,11 +380,11 @@ def Annotation(ids_sorted, annotation_df, annotation_col_id, axis = 1,
     min_val = None
     if(not is_categorial):
         max_val = max(
-            list(annotation_df.loc[ids_sorted, 
+            list(annotation_df.loc[ids_sorted,
                                    annotation_col_id].replace(np.nan, 0))
         )
         min_val = min(
-            list(annotation_df.loc[ids_sorted, 
+            list(annotation_df.loc[ids_sorted,
                                    annotation_col_id].replace(np.nan, 0))
         )
 
@@ -366,7 +397,7 @@ def Annotation(ids_sorted, annotation_df, annotation_col_id, axis = 1,
     else:
         color_counter = 0
         for group in groups:
-            groups_color_dict[group] = color_list[color_counter % 
+            groups_color_dict[group] = color_list[color_counter %
                                                   len(color_list)]
             color_counter += 1
 
@@ -461,33 +492,31 @@ def ColorScale(table,
         vmin = None,
         vmax = None,
         ax = None):
-    '''
-        Function that plots the color scale of values inside a dataframe.
+    """Function that plots the color scale of values inside a dataframe.
 
-        args:
-            table: pandas.DataFrame
-                Table containing numerical values.
+    :param table: Table containing numerical values.
+    :type table: :class:`pandas.DataFrame`
+    :param cmap: Colormap used to produce color scale, defaults to "Reds".
+    :type cmap: str, optional
+    :param symmetric_color_scale: If true, vmin, and vmax will be set to have
+        equal distance from symmetry point, defaults to True.
+    :type symmetric_color_scale: bool, optional
+    :param symmetry_point: Only used, when symmetric_color_scale is true. If
+        symmetric_color_scale is true, and symmetry_point is not set, defaults
+        to zero.
+    :type symmetry_point: float, optional
+    :param vmin: Minimal value of data_table, that has a color representation,
+        defaults to None.
+    :type vmin: float, optional
+    :param vmax: Maximal value of data_table, that has a color representation,
+        defaults to None.
+    :type vmax: float, optional
+    :param ax: Axes instance on which to plot the color scale, defaults to None.
+    :type ax: class:`matplotlib.axes._subplots.AxesSubplot`, optional
 
-        kwargs:
-            cmap: str
-                Colormap used to produce color scale.
-            symmetric_color_scale: bool
-                If true, vmin, and vmax will be set to have equal distance from 
-                symmetry point.
-            symmetry_point: float
-                Only used, when symmetric_color_scale is true. If
-                symmetric_color_scale is true, and symmetry_point is not set,
-                it defaults to zero.
-            vmin: float
-                Minimal value of data_table, that has a color representation.
-            vmax: float
-                Maximal value of data_table, that has a color representation.
-            ax: matplotlib.axes.Axes
-                Axes instance on which to plot the color scale.
-        Returns:
-            Nothing to be returned.
-
-    '''
+    :returns: Min. and max value displayed on color scale
+    :rtype: tuple
+    """
     ax = ax if ax is not None else plt.gca()
 
     # Calculate min and max value from table
@@ -518,27 +547,32 @@ def ColorScale(table,
 
     return vmin, vmax
 
-def Legends(patch_list_dict, annotation_ids = None, ax = None):
-    '''
-        Function that plots Legends based on pathc lists from Annotation
-        function.
+def Legends(patch_list_dict,
+            annotation_ids = None,
+            ax = None):
+    """Function that plots Legends based on pathc lists from Annotation
+    function.
 
-        args:
-            patch_list_dict: dict
-                A dictionary, storing patches used for legend plotting. The
-                key is the name of the Annotation, and the value is a list
-                of 2 elements. The first element is a boolean value, defining
-                if the annotation was catgorial (True) or non-categorial
-                (False). The second element is in the case of categorial
-                annotations the patch list, and in the case of non categorial
-                annotations the cmap used, as well as the min and the max
-                values.
-        kwargs:
-            ax: matplotlib.axes.Axes
-                Axes instance on which to plot the legends
-    '''
-    annotation_ids = (annotation_ids if 
-                      annotation_ids is not None else 
+    :param patch_list_dict: A dictionary, storing patches used for legend
+        plotting. The key is the name of the Annotation, and the value is a list
+        of 2 elements. The first element is a boolean value, defining if the
+        annotation was catgorial (True) or non-categorial (False). The second
+        element is in the case of categorial annotations the patch list, and in
+        the case of non categorial annotations the cmap used, as well as the min
+        and the max values.
+    :type patch_list_dict: dict
+    :param annotation_ids: List of annotation ids, for which the Legend shall
+        be plotted. Legends will be plotted in the same order as given in
+        annotation_ids list, defaults to None.
+    :type annotation_ids: list, optional
+    :param ax: Axes instance on which to plot the legends, defaults to None.
+    :type ax: class:`matplotlib.axes._subplots.AxesSubplot`, optional
+
+    :return: Nothing to be returned.
+    :rtype: None
+    """
+    annotation_ids = (annotation_ids if
+                      annotation_ids is not None else
                       patch_list_dict.keys())
     ax = ax if ax is not None else plt.gca()
 
@@ -595,8 +629,8 @@ def Legends(patch_list_dict, annotation_ids = None, ax = None):
             ax.axis("off")
         else:
             patch_list = patch_list_dict[annotation_id][1]
-            img = ax.imshow([[0, 1], [0, 1]], 
-                        cmap = patch_list[0], 
+            img = ax.imshow([[0, 1], [0, 1]],
+                        cmap = patch_list[0],
                         interpolation = 'bicubic',
                         extent=[x, x+color_scale_width, y, y-color_scale_height],
                         vmin=0
@@ -606,8 +640,8 @@ def Legends(patch_list_dict, annotation_ids = None, ax = None):
                 img.remove()
                 y = 1.-4.*(1./height)
                 x = x_max
-                img = ax.imshow([[0, 1], [0, 1]], 
-                        cmap = patch_list[0], 
+                img = ax.imshow([[0, 1], [0, 1]],
+                        cmap = patch_list[0],
                         interpolation = 'bicubic',
                         extent=[x, x+color_scale_width, y, y-color_scale_height],
                         vmin=0
@@ -621,14 +655,14 @@ def Legends(patch_list_dict, annotation_ids = None, ax = None):
                      fontsize=7,
                      verticalalignment="bottom",
                      horizontalalignment="center")
-            plt.text(x, 
-                     y-((.6)*(1./height)+color_scale_height), 
+            plt.text(x,
+                     y-((.6)*(1./height)+color_scale_height),
                      str(round(patch_list[1], 3)),
                      verticalalignment="top",
                      horizontalalignment="left",
                      fontsize=6)
-            plt.text(x+color_scale_width, 
-                     y-((.6)*(1./height)+color_scale_height), 
+            plt.text(x+color_scale_width,
+                     y-((.6)*(1./height)+color_scale_height),
                      str(round(patch_list[2], 3)),
                      verticalalignment="top",
                      horizontalalignment="right",
